@@ -4,7 +4,7 @@
 /*  Shows local time in different timezones + JD + Sidereal time +           */
 /*  internet time + local date and time.                                     */
 /*                                                                           */
-/*  Jan Lindblom <jlin@algonet.se> (http://www.algonet.se/~jlin/)            */
+/*  Jan Lindblom <99jl@home.se> (http://www.geocities.com/~jl1n/)            */
 /*                                                                           */
 /*  wmtz.c was derived from:                                                 */
 /*                                                                           */
@@ -37,37 +37,39 @@
 #include <X11/xpm.h>
 #include <X11/extensions/shape.h>
 
-#include "../wmgeneral/wmgeneral.h"
-#include "../wmgeneral/misc.h"
+#include <libdockapp/wmgeneral.h>
+#include <libdockapp/misc.h>
 #include "wmtz.xpm"
+#include "wmtz_mono.xpm"
 
 
 /*
-* Defines
-*/
-#define WMTZ_VERSION "0.6"
+ * Defines
+ */
+#define WMTZ_VERSION "0.7"
 #define CHAR_WIDTH 6
 #define STRSIZE 10
 #define LMST 1
 #define GMST 0
-#define ABOUT "xmessage -center -buttons \"Close\" \"WMTZ - Window Maker Time Zone dockapp.
-http://www.algonet.se/~jlin/wmtz/wmtz.html\""
+#define ABOUT "xmessage -center -buttons \"Close\" \"WMTZ - Window Maker Time Zone dockapp v0.7 http://www.geocities.com/jl1n/wmtz/wmtz.html\""
 
 
 /*
-* Typedefs
-*/
+ * Typedefs
+ */
 typedef struct
 {
-    char label[10]; /* Time zone designation */
-    int  diff;      /* Time zone diff. from UT in hours OR when label="JDN" */
-} timezone_t;       /* ...numbers of days to subtract from JD */
+    char label[10];    /* Time zone designation */
+    int  diff;         /* Time zone diff. from UT in hours */
+    double epoch;      /* Epoch (for JD) */
+    char   tz[256];     /* TZ environment variable string */
+} timezone_t;          /* ...numbers of days to subtract from JD */
 
 
 /*
-* Global variables
-*/
-timezone_t    zone[6]; 
+ * Global variables
+ */
+timezone_t    zone[6];
 double        longitude = 0.0;
 double        latitude = 0.0;
 static        struct tm *clk;
@@ -80,12 +82,13 @@ char          *week_day[7];
 char          wmtz_mask_bits[64*64];
 int           wmtz_mask_width = 64;
 int           wmtz_mask_height = 64;
+int           mono = 0;
 extern char   *tzname[2];
-
+static char   originalTZ[64];
 
 /*
-* Function declarations
-*/
+ * Function declarations
+ */
 void   usage(void);
 void   printversion(void);
 void   BlitString(char *name, int x, int y);
@@ -95,11 +98,13 @@ int    ReadConfigString(FILE *fp, char *setting, char *value);
 int    Read_Config_File(char *filename);
 void   range(double *val, double ran);
 void   siderealTime(double jde, int *result, int mode);
-double julianDay(int year, int month, double day, double hour, 
+double julianDay(int year, int month, double day, double hour,
                  double minute, double second, int julian);
 int    calendarDate(double jd, int *year, int *month, double *day);
 int    handleJD(void);
 void   errH(int printErrno, int exitCode, const char *msg, ...);
+void   handleTheMenu(int but_stat);
+double jdn(time_t curtime);
 
 
 /*****************************************************************************\
@@ -107,17 +112,31 @@ void   errH(int printErrno, int exitCode, const char *msg, ...);
 \*****************************************************************************/
 int main(int argc, char *argv[]) {
 
-  int		i; 
+  int		i;
+  char    *envbuf;
 
   /* Store away the executable name for error messages */
-  pname = strdup(argv[0]);
-  if (pname == NULL)        
-      errH(1, 0, "strdup");
-	
+  if ( (pname = strrchr( argv[0], '/' )) == NULL )
+      pname = argv[0];
+  else
+      pname++;
+
+  /* Store away the TZ environment variable, if set */
+  if ( (envbuf = getenv("TZ")) != NULL )
+  {
+    //Write TZ=envbuf into originalTZ
+    (void) sprintf(originalTZ, "TZ=%s", envbuf);
+  }
+  else
+  {
+    // Set originalTZ to TZ erase TZ env.
+    (void) sprintf(originalTZ, "TZ");
+  }
+
   /* Parse Command Line */
   for (i=1; i<argc; i++) {
     char *arg = argv[i];
-    
+
     if (*arg=='-') {
       switch (arg[1]) {
       case 'd' :
@@ -140,6 +159,9 @@ int main(int argc, char *argv[]) {
         handleJD();
         exit(0);
 	break;
+      case 'm' :
+	  mono = 1;
+	  break;
       case 'v' :
 	printversion();
 	exit(0);
@@ -147,9 +169,9 @@ int main(int argc, char *argv[]) {
       case 'c' :
 	if (argc > (i+1))
 	  {
-	    config_file = strdup(argv[i+1]);
-            if (config_file == NULL)
-                errH(1, 1, "strdup");  
+	    uconfig_file = strdup(argv[i+1]);
+            if (uconfig_file == NULL)
+                errH(1, 1, "strdup");
 	    i++;
 	  }
 	break;
@@ -174,7 +196,7 @@ int main(int argc, char *argv[]) {
   month[3] = "APR";  month[4] = "MAY";  month[5] = "JUN";
   month[6] = "JUL";  month[7] = "AUG";  month[8] = "SEP";
   month[9] = "OUT";  month[10] = "NOV";  month[11] = "DEC";
-  
+
   week_day[0] = "SUNDAY   ";
   week_day[1] = "MONDAY   ";
   week_day[2] = "TUESDAY  ";
@@ -182,7 +204,7 @@ int main(int argc, char *argv[]) {
   week_day[4] = "THURSDAY ";
   week_day[5] = "FRIDAY   ";
   week_day[6] = "SATURDAY ";
-  
+
   wmtz_routine(argc, argv);
 
   return 0;
@@ -200,18 +222,28 @@ void wmtz_routine(int argc, char **argv)
     float               swatch_beats;
     time_t		curtime;
     time_t		prevtime;
-    char                *editor;
-    char                *ed;
     char                *home;
     char                buf[64];
     char                blitstr[STRSIZE];
     XEvent              Event;
 
-    createXBMfromXPM(wmtz_mask_bits, wmtz_master_xpm, 
-                     wmtz_mask_width, wmtz_mask_height);
-    
-    openXwindow(argc, argv, wmtz_master_xpm, wmtz_mask_bits, 
-                wmtz_mask_width, wmtz_mask_height);
+    if (mono)
+    {
+	fprintf(stderr, "Starting monochrome version.\n");
+	createXBMfromXPM(wmtz_mask_bits, wmtz_mono_xpm,
+			 wmtz_mask_width, wmtz_mask_height);
+
+	openXwindow(argc, argv, wmtz_mono_xpm, wmtz_mask_bits,
+		    wmtz_mask_width, wmtz_mask_height);
+    }
+    else
+    {
+	createXBMfromXPM(wmtz_mask_bits, wmtz_master_xpm,
+			 wmtz_mask_width, wmtz_mask_height);
+
+	openXwindow(argc, argv, wmtz_master_xpm, wmtz_mask_bits,
+		    wmtz_mask_width, wmtz_mask_height);
+    }
 
     memset(&zone, 0, sizeof(zone));
 
@@ -225,7 +257,7 @@ void wmtz_routine(int argc, char **argv)
     if (uconfig_file != NULL)
     {
         /* user-specified config file */
-        fprintf(stderr, "Using user-specified config file '%s'.\n", 
+        fprintf(stderr, "Using user-specified config file '%s'.\n",
                 uconfig_file);
         config_file = strdup(uconfig_file);
         free(uconfig_file);
@@ -249,19 +281,19 @@ void wmtz_routine(int argc, char **argv)
                 errH(1, 1, "malloc");
 
             sprintf(config_file, "/etc/wmtzrc");
-            
+
             fprintf(stderr, "Using /etc/wmtzrc as config file.\n");
-        
+
             Read_Config_File(config_file);
         }
     }
 
     RedrawWindow();
     prevtime = time(0) - 1;
-    
+
     while (1)
     {
-        curtime = time(0);
+	waitpid(0, NULL, WNOHANG);
 
         /* If wmtz have been mouse-clicked, show menu */
         if ( clicked )
@@ -272,7 +304,7 @@ void wmtz_routine(int argc, char **argv)
             BlitString(" RESTART ", 5, (11*(3)) + 5);
             BlitString(" QUIT    ", 5, (11*(4)) + 5);
 	}
-        else if ( curtime > prevtime)
+        else if ( (curtime = time(0)) > prevtime)
         {
 	  prevtime = curtime;
 
@@ -282,7 +314,7 @@ void wmtz_routine(int argc, char **argv)
             /* Display empty line */
             if (strncmp( zone[j].label, "xxx", 3) == 0 )
 	    {
-              BlitString("        ", 5, (11*(j-1)) + 5); 
+              BlitString("        ", 5, (11*(j-1)) + 5);
 	    }
 
 	    /* Display local day/mon/year */
@@ -292,7 +324,7 @@ void wmtz_routine(int argc, char **argv)
 	      while(clk->tm_year>99) clk->tm_year-=100;
 	      snprintf(blitstr, STRSIZE,"%s %02d.%02d",
                        month[clk->tm_mon],clk->tm_mday,clk->tm_year);
-              BlitString(blitstr, 5, (11*(j-1)) + 5);  
+              BlitString(blitstr, 5, (11*(j-1)) + 5);
 	    }
 
 	    /* Display local weekday */
@@ -300,7 +332,7 @@ void wmtz_routine(int argc, char **argv)
 	    {
 	      clk = localtime(&curtime);
 	      snprintf(blitstr, STRSIZE,"%s",week_day[clk->tm_wday]);
-	      BlitString(blitstr, 4, (11*(j-1)) + 5);  
+	      BlitString(blitstr, 4, (11*(j-1)) + 5);
 	    }
 
 	    /* Display more precise internet time */
@@ -317,7 +349,7 @@ void wmtz_routine(int argc, char **argv)
 	    {
  	      clk = gmtime(&curtime);
 	      /* Calculate Julin Day Number */
-	      jd = curtime/86400.0 + 2440587.5 - zone[j].diff;
+	      jd = jdn(curtime) - zone[j].epoch;
 	      snprintf(blitstr, STRSIZE, "%10f", jd );
 	      BlitString(blitstr, 5, (11*(j-1)) + 5);
 	    }
@@ -326,7 +358,7 @@ void wmtz_routine(int argc, char **argv)
             else if (strncmp( zone[j].label, "LMST", 3) == 0 )
 	    {
               clk = gmtime(&curtime);
-              jd = curtime/86400.0 + 2440587.5;
+              jd = jdn(curtime);
               siderealTime( jd, sid, LMST );
               snprintf(blitstr, STRSIZE, "%s%02i.%02i","LST:", sid[0], sid[1]);
               BlitString(blitstr, 5, (11*(j-1)) + 5);
@@ -336,7 +368,7 @@ void wmtz_routine(int argc, char **argv)
             else if (strncmp( zone[j].label, "GMST", 3) == 0 )
 	    {
               clk = gmtime(&curtime);
-              jd = curtime/86400.0 + 2440587.5;
+              jd = jdn(curtime);
               siderealTime( jd, sid, GMST );
               snprintf(blitstr, STRSIZE, "%s%02i.%02i","GST:", sid[0], sid[1]);
               BlitString(blitstr, 5, (11*(j-1)) + 5);
@@ -347,7 +379,7 @@ void wmtz_routine(int argc, char **argv)
 	    {
 	      clk = localtime(&curtime);
 	      strncpy(buf, tzname[0], 3);
-	      
+
               for (k=0; k<3; k++)
                 if (buf[k] == 0)
                   buf[k] = ' ';
@@ -355,10 +387,6 @@ void wmtz_routine(int argc, char **argv)
               buf[3] = ':';
               buf[4] = 0;
 	      hour = clk->tm_hour;
-              if (hour > 23 )
-                hour -= 24;
-              else if (hour < 0 )
-                hour += 24;
 
 	      /* Print Label */
               snprintf(blitstr, STRSIZE, "%s%02i.%02i",buf,hour,clk->tm_min);
@@ -366,11 +394,35 @@ void wmtz_routine(int argc, char **argv)
 	    }
 
             /* Display time in specified time zone */
-            else 
+	    else if (strncmp( zone[j].label, "TZONE", 4) == 0 )
+	    {
+	      putenv(zone[j].tz);
+	      tzset();
+	      clk = localtime(&curtime);
+
+	      strncpy(buf, tzname[0], 3);
+
+              for (k=0; k<3; k++)
+                if (buf[k] == 0)
+                  buf[k] = ' ';
+
+              buf[3] = ':';
+              buf[4] = 0;
+
+              snprintf(blitstr, STRSIZE, "%s%02i.%02i", buf,
+		       clk->tm_hour, clk->tm_min);
+              BlitString(blitstr, 5, (11*(j-1)) + 5);
+
+	      /* Reset TZ environment variable to old value */
+	      putenv(originalTZ);
+	    }
+
+            /* Display time in specified time zone without TZ env. var. */
+            else
             {
  	      clk = gmtime(&curtime);
               strncpy(buf, zone[j].label, 3);
-                       
+
               for (k=0; k<3; k++)
                 if (buf[k] == 0)
                   buf[k] = ' ';
@@ -379,16 +431,16 @@ void wmtz_routine(int argc, char **argv)
               buf[4] = 0;
 
               hour = clk->tm_hour - zone[j].diff;
-              if (hour > 23 ) 
+              if (hour > 23 )
 		hour -= 24;
-              else if (hour < 0 ) 
+              else if (hour < 0 )
 		hour += 24;
 
 	      /* Print Label */
               snprintf(blitstr, STRSIZE, "%s%02i.%02i",buf,hour,clk->tm_min);
               BlitString(blitstr, 5, (11*(j-1)) + 5);
 	    }
-          }                                     
+          }
         }
         RedrawWindow();
 
@@ -415,44 +467,8 @@ void wmtz_routine(int argc, char **argv)
                    {
 		     if ( clicked ) /* The menu is up */
 		     {
-		       switch( but_stat )
-		       {
-		         case 0:
-                           waitpid( execCommand(ABOUT), NULL, 0); 
-			   clicked = 0;
-			   break;
-		         case 1:
-			   clicked = 0;
-			   break;
-		         case 2:
-			   /* Figure out what editor to use */
-                           if ( defedit == NULL )
-                           {
-			     ed = getenv("XEDITOR"); 
-                             if ( ed == NULL ) 
-                                 ed = "xedit";
-                           }
-                           else
-                           {
-                             ed = defedit;
-                           }
-                           editor = malloc( strlen(ed)+strlen(config_file)+2 );
-			   if ( editor == NULL )
-			       errH(1, 1, "malloc");
-                           sprintf(editor, "%s %s", ed, config_file);
-                           waitpid( execCommand(editor), NULL, 0 );
-                           Read_Config_File(config_file);
-                           free(editor);
-			   clicked = 0;
-			   break;
-		         case 3:
-                           Read_Config_File(config_file);
-			   clicked = 0;
-			   break;
-		         case 4:
-			   exit(0);
-			   break;
-		       }
+		         handleTheMenu(but_stat);
+		         clicked = 0;
 		     }
                      else /* Show the menu instead of time */
 		     {
@@ -461,11 +477,57 @@ void wmtz_routine(int argc, char **argv)
                    }
 		   but_stat = -1;
                    break;
-	    }
-        }  
+	  }
+        }
 
-        usleep(100000);
-    }        
+        usleep(10000);
+    }
+}
+
+
+/*****************************************************************************\
+|* handleTheMenu                                                             *|
+\*****************************************************************************/
+void handleTheMenu(int but_stat)
+{
+    char                *editor;
+    char                *ed;
+
+    switch( but_stat )
+      {
+	case 0:
+	    execCommand(ABOUT);
+	    break;
+	case 1:
+	    break;
+	case 2:
+	    /* Figure out what editor to use */
+	    if ( defedit == NULL )
+		{
+		    ed = getenv("XEDITOR");
+		    if ( ed == NULL )
+			ed = "xedit";
+		}
+	    else
+		{
+		    ed = defedit;
+		}
+	    editor = malloc( strlen(ed)+strlen(config_file)+2 );
+	    if ( editor == NULL )
+		errH(1, 1, "malloc");
+	    sprintf(editor, "%s %s", ed, config_file);
+	    waitpid( execCommand(editor), NULL, 0 );
+	    Read_Config_File(config_file);
+	    free(editor);
+	    break;
+	case 3:
+	    Read_Config_File(config_file);
+	    break;
+	case 4:
+	    exit(0);
+	    break;
+      }
+    return;
 }
 
 
@@ -488,15 +550,15 @@ int ReadConfigString(FILE *fp, char *setting, char *value)
 
     snprintf(str,1024, "%s=", setting);
     slen = strlen(str);
-    
+
     fseek(fp, 0, SEEK_SET);
 
     while ( !feof(fp) )
     {
-        
+
         if (!fgets(buf, 512, fp))
 	  break;
-        
+
         len = strlen(buf);
 
         /* strip linefeed */
@@ -508,7 +570,7 @@ int ReadConfigString(FILE *fp, char *setting, char *value)
         if ( strncmp(buf, str, strlen(str)) == 0)
         {
             /* found our setting */
-	  
+
 	  for(i=0; i!=slen; i++)
 	    if ( buf[i] == '=' )
 	      {
@@ -517,7 +579,7 @@ int ReadConfigString(FILE *fp, char *setting, char *value)
 		return 1;
 	      }
         }
-    }    
+    }
     return 0;
 }
 
@@ -562,6 +624,7 @@ int ReadConfigDouble(FILE *fp, char *setting, double *value)
 int Read_Config_File( char *filename )
 {
     FILE *fp;
+    char temp[253];
 
     fp = fopen(filename, "r");
     if (fp)
@@ -576,8 +639,24 @@ int Read_Config_File( char *filename )
         ReadConfigInt(fp, "utdiff3", &zone[3].diff);
         ReadConfigInt(fp, "utdiff4", &zone[4].diff);
         ReadConfigInt(fp, "utdiff5", &zone[5].diff);
+        ReadConfigDouble(fp, "utdiff1", &zone[1].epoch);
+        ReadConfigDouble(fp, "utdiff2", &zone[2].epoch);
+        ReadConfigDouble(fp, "utdiff3", &zone[3].epoch);
+        ReadConfigDouble(fp, "utdiff4", &zone[4].epoch);
+        ReadConfigDouble(fp, "utdiff5", &zone[5].epoch);
+
+        ReadConfigString(fp, "utdiff1", temp);
+	sprintf(zone[1].tz, "TZ=%s", temp);
+        ReadConfigString(fp, "utdiff2", temp);
+	sprintf(zone[2].tz, "TZ=%s", temp);
+        ReadConfigString(fp, "utdiff3", temp);
+	sprintf(zone[3].tz, "TZ=%s", temp);
+        ReadConfigString(fp, "utdiff4", temp);
+	sprintf(zone[4].tz, "TZ=%s", temp);
+        ReadConfigString(fp, "utdiff5", temp);
+	sprintf(zone[5].tz, "TZ=%s", temp);
+
         ReadConfigDouble(fp, "longitude", &longitude);
-        /* Not needed, actually... */
         /* ReadConfigDouble(fp, "latitude", &latitude); */
 
         fclose(fp);
@@ -604,7 +683,7 @@ void BlitString(char *name, int x, int y)
     for (i=0; name[i]; i++)
     {
 
-        c = toupper(name[i]); 
+        c = toupper(name[i]);
 	if (c >= 'A' && c <= 'Z')
 	{   /* its a letter */
 	  c -= 'A';
@@ -612,7 +691,7 @@ void BlitString(char *name, int x, int y)
 	  k += CHAR_WIDTH;
         }
         else if ( c >= '0' && c <= ':')
-        {   
+        {
 	  c -= '0';
 	  copyXPMArea(c * CHAR_WIDTH, 64, CHAR_WIDTH, 8, k, y);
 	  k += CHAR_WIDTH;
@@ -623,7 +702,7 @@ void BlitString(char *name, int x, int y)
 	  k += 4;
 	}
 	else if (c=='.')
-	  { 
+	  {
 	  copyXPMArea(115, 64, 4, 8, k, y);
 	  k += 4;
 	}
@@ -633,7 +712,7 @@ void BlitString(char *name, int x, int y)
 	  k += CHAR_WIDTH;
 	}
 	else /* print a ' ' */
-	{  
+	{
 	  copyXPMArea(120, 64, CHAR_WIDTH, 8, k, y);
 	  k += CHAR_WIDTH;
 	}
@@ -650,10 +729,11 @@ void usage(void)
   fprintf(stderr, "See ~/.wmtzrc or /etc/wmtzrc for configuration.\n\n");
   fprintf(stderr, "Usage:\n");
   fprintf(stderr, "    -display <display name>\n");
-  fprintf(stderr, "    -e <editor>               use specified editor\n");   
+  fprintf(stderr, "    -e <editor>               use specified editor\n");
   fprintf(stderr, "    -geometry +XPOS+YPOS      initial window position\n");
   fprintf(stderr, "    -jd                       Julian<->Date conversion\n");
   fprintf(stderr, "    -c <filename>             use specified config file\n");
+  fprintf(stderr, "    -m                        start monochrome version\n");
   fprintf(stderr, "    -h                        this help screen\n");
   fprintf(stderr, "    -v                        print the version number\n");
   fprintf(stderr, "\n");
@@ -675,12 +755,20 @@ void printversion(void)
 void range (double *val, double ran)
 {
       *val -= ran*floor(*val/ran);
-       
-      if (*val < 0) 
+
+      if (*val < 0)
           *val += ran;
 }
 
 
+/*****************************************************************************\
+|* jdn - converts a time_t to Julian Day                                     *|
+\*****************************************************************************/
+double jdn(time_t curtime)
+{
+    return (curtime/86400.0 + 2440587.5);
+}
+
 /*****************************************************************************\
 |* siderealTime - Gives sidereal time from JD.                               *|
 \*****************************************************************************/
@@ -696,7 +784,7 @@ void siderealTime( double jde, int *result, int mode )
    ts = 280.46061837 + 360.98564736629 * ( jde - 2451545.0 )
         + 0.000387933 * t2 - t3/38710000.0;
 
-   range( &ts, 360.0 ); 
+   range( &ts, 360.0 );
    ts /= 15.0;
 
    /* If local time add one hour for every 15 degree in longitude */
@@ -714,7 +802,7 @@ void siderealTime( double jde, int *result, int mode )
 /*****************************************************************************\
 |* julianDay - Gives JD from date.                                           *|
 \*****************************************************************************/
-double julianDay( int year, int month, double day, double hour, 
+double julianDay( int year, int month, double day, double hour,
                   double minute, double second, int julian )
 {
    int a, b, c, d;
@@ -755,17 +843,17 @@ int calendarDate( double jd, int *year, int *month, double *day )
 
    if ( jd < 0 )
       return 0;
-    
+
    jd += 0.5;
    ij = floor(jd);
    frac = jd - ij;
 
-   if ( ij < 2299161 ) 
+   if ( ij < 2299161 )
    {
       a = ij;
    }
-   else 
-   { 
+   else
+   {
       alfa = floor((ij - 1867216.25)/36524.25);
       beta = floor(alfa/4);
       a = ij + 1 + alfa - beta;
@@ -783,7 +871,7 @@ int calendarDate( double jd, int *year, int *month, double *day )
        *month = e - 1;
    else if (e == 14 || e == 15)
        *month = e - 13;
-   else 
+   else
        return 0;
 
    if (*month > 2)
@@ -792,7 +880,7 @@ int calendarDate( double jd, int *year, int *month, double *day )
        *year = c - 4715;
    else
        return 0;
-  
+
    return 1;
 }
 
@@ -813,7 +901,7 @@ int handleJD( void )
   {
      printf("Enter UT date with time (YYYY,MM,DD,hh:mm:ss): ");
      scanf("%d,%d,%d,%d:%d:%d", &y, &m, &d, &h, &min, &sec);
-     printf("\nJulian Day: %f\n", julianDay( y, m, d, h, min, sec, 0 ) ); 
+     printf("\nJulian Day: %f\n", julianDay( y, m, d, h, min, sec, 0 ) );
   }
   else if (conv == 2)
   {
@@ -824,7 +912,7 @@ int handleJD( void )
        printf("Conversion error! Negative JD not allowed.\n");
        return 0;
      }
-       
+
      printf("\nGregorian date: %d-%2.2d-%2.4f\n", y, m, day);
   }
   else
@@ -848,14 +936,15 @@ void errH(int printErrno, int exitCode, const char *msg, ...)
     buf[0] = 0;
     strcat(buf, pname);
     strcat(buf, ": ");
-    
+
     va_start(arg, msg);
     vsprintf(buf+strlen(buf), msg, arg);
     if (printErrno)
-        sprintf(buf+strlen(buf), ": %s\n", strerror(error));
-    fflush(stdout); 
+        sprintf(buf+strlen(buf), ": %s", strerror(error));
+    strcat(buf, "\n");
+    fflush(stdout);
     fputs(buf, stderr);
-    fflush(NULL); 
+    fflush(NULL);
     va_end(arg);
 
     if ( exitCode )
